#include "StdAfx.h"
#include "CsvFile.h"
#define CSV_SEPARATOR 44

CCsvFile::CCsvFile()
{
	m_bHasHeader = FALSE;
}

CCsvFile::~CCsvFile()
{
	Close();
}

BOOL CCsvFile::IsOpen()
{
	return (CFile::hFileNull != m_File.m_hFile);
}

BOOL CCsvFile::Open(LPCTSTR szName, CString &sError, BOOL bHasHeader)
{
//  checking if the file is open
	if (IsOpen())
	{
		sError = _T("A file is already open");
		return FALSE;
	}

//  trying to open the file
	CFileException theException;
	if (!m_File.Open(szName, CFile::modeRead | CFile::shareDenyNone, &theException))
	{
		TCHAR szError[256];
		theException.GetErrorMessage(szError, 256);
		sError = szError;
	}

//  skipping file header if any
	CString sDummy;
	if (bHasHeader)
	{
		m_File.ReadString(sDummy);
		m_bHasHeader = TRUE;
	}

//  exiting
	return IsOpen();
}

void CCsvFile::Close()
{
	if (IsOpen())
	{
		m_File.Close();
		m_sRecord.Empty();
	}
}

BOOL CCsvFile::Rewind(CString &sError)
{
//  checking if the file is open
	if (!IsOpen())
	{
		sError = _T("Please open the file first");
		return FALSE;
	}

//  trying to move the internal CStdioFile pointer
	m_sRecord.Empty();
	try
	{
	//  miving file pointer
		ULONGLONG ullPosition = m_File.Seek(0, CFile::begin);

	//  skipping file header if any
		CString sDummy;
		if (m_bHasHeader)
			m_File.ReadString(sDummy);

	//  exiting
		return (0 == ullPosition);
	}
	catch (CFileException *e)
	{
		TCHAR szError[256];
		e->GetErrorMessage(szError, 256);
		sError = szError;
		return FALSE;
	}
}

BOOL CCsvFile::Next(CString &sError)
{
//  checking if the file is open
	if (!IsOpen())
	{
		sError = _T("Please open the file first");
		return FALSE;
	}

//  reading the next CSV file line
	m_sRecord.Empty();
	return m_File.ReadString(m_sRecord);
}

CString CCsvFile::GetField(int index)
{
//  if the CSV file record data is not loaded, return nothing
	if (m_sRecord.IsEmpty())
		return _T("");

//  looking for the beginning of the requierd field in the record string
	int iStart = 0, iStop = m_sRecord.GetLength()-1;
	for (int i=0; i<index; i++)
	{
		iStart = m_sRecord.Find(CSV_SEPARATOR, iStart);
		if (-1 == iStart)
			return _T("");
		else
			iStart++;
	}

//  looking for the end of the requierd field in the record string 
	iStop = m_sRecord.Find(CSV_SEPARATOR, iStart);
	if (-1 == iStop)
		iStop = m_sRecord.GetLength();
	else
		iStop--;

//  returning the value found
	if (iStop > iStart)
		return m_sRecord.Mid(iStart, iStop-iStart+1);
	else
		return _T("");
}