#include "StdAfx.h"
#include "Oxetta.h"
#include "MemEmployeeList.h"
#include "CsvFile.h"
#include "..\HelperFunctions.h"

typedef struct
{
	CCsvFile m_csvDepartments;
	CCsvFile m_csvEmployee;
} CCsvDatabase;

int OnPrintGroupHeader(TCHAR *szName, TCHAR *szValue, void **ppParam)
{
//  getting a pointer to the data files
	CString sError;
	ASSERT( ppParam);
	CCsvDatabase *pDatabase = (CCsvDatabase*)(*ppParam);
	ASSERT(pDatabase);
	ASSERT(pDatabase->m_csvDepartments.IsOpen());

//  if the reporting engine have passed NULL in the element name, then it wants to know if we want to print the entire group header band
	if (NULL == szName)
	{
		pDatabase->m_csvEmployee.Rewind(sError);
		return pDatabase->m_csvDepartments.Next(sError);
	}
	else
	if (0 == _tcsicmp(szName, _T("Department")))
	{
		_tcscpy(szValue, pDatabase->m_csvDepartments.GetField(1));
		return TRUE;
	}
	else
		return TRUE;
}

int OnPrintContents(TCHAR *szName, TCHAR *szValue, void **ppParam)
{
//  getting a pointer to the data files
	ASSERT( ppParam);
	CCsvDatabase *pDatabase = (CCsvDatabase*)(*ppParam);
	ASSERT(pDatabase);
	ASSERT(pDatabase->m_csvDepartments.IsOpen());

//  if the reporting engine have passed NULL in the element name, then it wants to know if we want to print the entire contents band
	CString sError;
	if (NULL == szName)
	{
	//  finding out what department we are currently printing
		int iDepartmentID = _ttoi(pDatabase->m_csvDepartments.GetField(0));
		if (0 == iDepartmentID)
			return FALSE;
	//  looking for the next employee in this department
		while (pDatabase->m_csvEmployee.Next(sError))
			if (_ttoi(pDatabase->m_csvEmployee.GetField(3)) == iDepartmentID)
				return TRUE;
	//  skipped to the end of file, and no personnel in this department found
		return FALSE;
	}
	else
	if (0 == _tcsicmp(szName, _T("Name")))
	{
		_tcscpy(szValue, pDatabase->m_csvEmployee.GetField(1) + _T(" ") + pDatabase->m_csvEmployee.GetField(2));
		return TRUE;
	}
	else
	if (0 == _tcsicmp(szName, _T("Salary")))
	{
		_tcscpy(szValue, pDatabase->m_csvEmployee.GetField(5));
		return TRUE;
	}
	else
		return TRUE;
}

int OnPrintPageFooter(TCHAR *szName, TCHAR *szValue, void **ppParam)
{
	static int iPageNo = 0;
	if (
		(NULL != szName) &&
		(0 == _tcsicmp(szName, _T("Page Number")))
	   )
		_stprintf(szValue, _T("page: %d"), ++iPageNo);
	return TRUE;
}

int PASCAL _stdcall EmployeeListCallback(TCHAR *szBand, TCHAR *szName, TCHAR *szValue, void **ppParam)
{
//  opening or getting a pointer to the data files
	CString sError;
	CCsvDatabase *pDatabase = (CCsvDatabase*)(*ppParam);

//  analyzing which report band initiated this callback
	if (0 == _tcsicmp(szBand, _T("Report Header")))
		return TRUE;
	else
	if (0 == _tcsicmp(szBand, _T("Group Header")))
		return OnPrintGroupHeader(szName, szValue, ppParam);
	else
	if (0 == _tcsicmp(szBand, _T("Report Contents")))
		return OnPrintContents(szName, szValue, ppParam);
	else
	if (0 == _tcsicmp(szBand, _T("Page Footer")))
		return OnPrintPageFooter(szName, szValue, ppParam);
	else
		return FALSE;

}

BOOL MemEmployeeList(CString &sError)
{
//  loading reporting library
	HMODULE hDll = LoadLibrary(GetStartDir() + _T("\\oxetta.dll"));
	if (NULL == hDll)
	{
		sError = _T("Failed to load reporting engine DLL");
		return FALSE;
	}

//  getting a pointer to the print preview function
#ifdef _UNICODE
	COxiPreview pfnPreview = (COxiPreview)GetProcAddress(hDll, "oxiPreviewW");
#else
	COxiPreview pfnPreview = (COxiPreview)GetProcAddress(hDll, "oxiPreviewA");
#endif
	if (NULL == pfnPreview)
	{
		FreeLibrary(hDll);
		hDll = NULL;
		sError = _T("Failed to get a pointer to a function in the reporting engine DLL");
		return FALSE;
	}

//  assembling the report file path string
	TCHAR szReport[MAX_PATH];
	_tcscpy(szReport, GetStartDir());
	_tcscat(szReport, _T("\\memEmployeeList.oxi"));

//  opening the CSV files database
	CCsvDatabase *pDatabase = new CCsvDatabase;
	if (!pDatabase->m_csvDepartments.Open(GetStartDir() + _T("\\departments.csv"), sError, TRUE))
	{
		delete pDatabase;
		pDatabase = NULL;
		return FALSE;
	}
	if (!pDatabase->m_csvEmployee.Open(GetStartDir() + _T("\\employee.csv"), sError, TRUE))
	{
		delete pDatabase;
		pDatabase = NULL;
		return FALSE;
	}


//  printing the report
	TCHAR szError[1024];
	BOOL bResult = pfnPreview(szReport, EmployeeListCallback, (void**)&pDatabase, szError);
	
//  cleaning up and exiting
	delete pDatabase;
	pDatabase = NULL;
	pfnPreview = NULL;
	FreeLibrary(hDll);
	hDll = NULL;
	sError = szError;
	return bResult;
}
